import { SqlInterfaceInstance, Request as NativeRequest, LoadRequestMeta } from '@cubejs-backend/native';
import type { ShutdownMode } from '@cubejs-backend/native';
import type { ApiGateway } from './gateway';
import type { CheckSQLAuthFn, ExtendedRequestContext, CanSwitchSQLUserFn } from './interfaces';
export type SQLServerOptions = {
    checkSqlAuth?: CheckSQLAuthFn;
    canSwitchSqlUser?: CanSwitchSQLUserFn;
    sqlPort?: number;
    pgSqlPort?: number;
    sqlUser?: string;
    sqlSuperUser?: string;
    sqlPassword?: string;
    gatewayPort?: number;
};
export type SQLServerConstructorOptions = {
    gatewayPort?: number;
};
export declare class SQLServer {
    protected readonly apiGateway: ApiGateway;
    protected sqlInterfaceInstance: SqlInterfaceInstance | null;
    protected readonly gatewayPort: number | undefined;
    constructor(apiGateway: ApiGateway, options: SQLServerConstructorOptions);
    getNativeGatewayPort(): number;
    execSql(sqlQuery: string, stream: any, securityContext?: any): Promise<void>;
    protected buildCheckSqlAuth(options: SQLServerOptions): CheckSQLAuthFn;
    init(options: SQLServerOptions): Promise<void>;
    protected wrapCheckSqlAuthFn(checkSqlAuth: CheckSQLAuthFn): CheckSQLAuthFn;
    protected createDefaultCanSwitchSqlUserFn(options: SQLServerOptions): CanSwitchSQLUserFn;
    protected createDefaultCheckSqlAuthFn(options: SQLServerOptions): CheckSQLAuthFn;
    protected contextByNativeReq(req: NativeRequest<LoadRequestMeta>, securityContext: any, requestId: string): Promise<ExtendedRequestContext>;
    close(): Promise<void>;
    shutdown(mode: ShutdownMode): Promise<void>;
}
//# sourceMappingURL=sql-server.d.ts.map